/* -----------------------------------------------------------------------------
 * Copyright 2022 Massachusetts Institute of Technology.
 * All Rights Reserved
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice,
 *     this list of conditions and the following disclaimer in the documentation
 *     and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Research was sponsored by the United States Air Force Research Laboratory and
 * the United States Air Force Artificial Intelligence Accelerator and was
 * accomplished under Cooperative Agreement Number FA8750-19-2-1000. The views
 * and conclusions contained in this document are those of the authors and should
 * not be interpreted as representing the official policies, either expressed or
 * implied, of the United States Air Force or the U.S. Government. The U.S.
 * Government is authorized to reproduce and distribute reprints for Government
 * purposes notwithstanding any copyright notation herein.
 * -------------------------------------------------------------------------- */
#include <pybind11/pybind11.h>
#include <spark_dsg/serialization/versioning.h>

#include "spark_dsg/python/attributes.h"
#include "spark_dsg/python/bounding_box.h"
#include "spark_dsg/python/color.h"
#include "spark_dsg/python/graph_types.h"
#include "spark_dsg/python/mesh.h"
#include "spark_dsg/python/metadata.h"
#include "spark_dsg/python/python_types.h"
#include "spark_dsg/python/scene_graph.h"
#include "spark_dsg/python/scene_graph_iterators.h"
#include "spark_dsg/python/scene_graph_layer.h"
#include "spark_dsg/python/spark_types.h"

PYBIND11_MODULE(_dsg_bindings, m) {
  pybind11::options options;

  spark_dsg::python::init_attributes(m);
  spark_dsg::python::init_bounding_box(m);
  spark_dsg::python::init_color(m);
  spark_dsg::python::init_graph_types(m);
  spark_dsg::python::init_mesh(m);
  spark_dsg::python::init_metadata(m);
  spark_dsg::python::init_python_types(m);
  spark_dsg::python::init_scene_graph(m);
  spark_dsg::python::init_scene_graph_layer(m);
  spark_dsg::python::init_spark_types(m);

  m.def("version", []() { return spark_dsg::io::FileHeader::current().version.toString(); });
}
